# -*- encoding: utf-8 -*-
#
# Copyright 2014 Telefonica Investigación y Desarrollo, S.A.U
#
# This file is part of FI-WARE project.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#
# For those usages not covered by the Apache version 2.0 License please
# contact with opensource@tid.es
#
__author__ = 'fla'



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2014 Telefónica Investigación y Desarrollo, S.A.U
#
# This file is part of FI-WARE project.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#
# For those usages not covered by the Apache version 2.0 License please
# contact with opensource@tid.es
#

__description__ = 'Facts Listener'
__author__ = 'fla'

from flask import Flask, request, Response, json
from facts.myredis import myredis
from facts.queue import myqueue
from facts.jsoncheck import jsoncheck
from gevent.pywsgi import WSGIServer
from keystoneclient.exceptions import NotFound
from facts.config import fact_attributes, __version__
from facts import cloto_db_client
from facts.constants import CONTENT_HEADER, JSON_TYPE, REMOTE_ADDR, REMOTE_PORT, CONTEXT_ATTRIBUTES, \
    CONTEXT_RESPONSES, CONTEXT_ATTRIBUTES_NAME, CONTEXT_ATTRIBUTES_VALUE, CONTEXT_ELEMENT
import logging.config
import sys
import datetime
import gevent.monkey
import os
import httplib
import gevent

__version_info__ = tuple([int(num) for num in __version__.split('.')])


gevent.monkey.patch_all()

content_type = JSON_TYPE

from facts.config import config, cfg_filename, cfg_defaults

"""Flask server initialization.

Uses Redis server as a message queue and server exchange with the fiware-cloto.
"""
app = Flask(__name__)

"""
Initialize the redis connection library
"""
mredis = myredis()

"""
Initialize the mysql connection library
"""

myClotoDBClient = cloto_db_client.cloto_db_client()

"""
Initialize the pid of the process
"""
pid = 0

# Flask/Gevent server need to send {'serverId': 'serverId', 'cpu': 80, 'mem': 80, 'time': '2014-03-24 16:21:29.384631'}
# to the topic


@app.route('/v1.0', methods=['GET'])
def factsinfo():
    """API endpoint for receiving keep alive information
    """
    return Response(response="{\"fiware-facts\":\"Up and running...\"}\n",
                    status=httplib.OK,
                    content_type=content_type)


@app.route('/v1.0/<tenantid>/servers/<serverid>', methods=['POST'])
def facts(tenantid, serverid):
    """API endpoint for receiving data from Monitoring system

    :param string tenantid:    the id of the tenant
    :param string serverid:    the id of the monitored instance (server)

    :return: status code 405 - invalid JSON or invalid request type
    :return: status code 400 - unsupported Content-Type or invalid publisher
    :return: status code 200 - successful submission
    """
    # Ensure post's Content-Type is supported
    if request.headers[CONTENT_HEADER] == content_type:
        try:
            # Ensure that received data is a valid JSON
            user_submission = json.loads(request.data)  # @UnusedVariable
        except ValueError:
            # Data is not a well-formed json
            message = "[{}] received {} from ip {}:{}"\
                .format("-", json, request.environ[REMOTE_ADDR], request.environ[REMOTE_PORT])

            logging.warning(message)

            return Response(response="{\"error\":\"Bad request. The payload is not well-defined json format\"}\n",
                            status=httplib.BAD_REQUEST,
                            content_type=content_type)

        # It is a valid payload and we start to process it
        try:
            result = process_request(request, tenantid, serverid)
        except NotFound as ex:
            return Response(response=ex.message, status=ex.http_status, content_type=content_type)
        except UnboundLocalError as ex:
            return Response(response="{\"error\":\"Some attribute is missing: " + ex.message + "\"}\n",
                            status=httplib.BAD_REQUEST, content_type=content_type)
        except ValueError as ex:
            return Response(response="{\"error\":\""+ ex.message + "\"}\n",
                            status=httplib.BAD_REQUEST, content_type=content_type)
        except Exception as ex:
            return Response(response="{\"error\": \"" + ex.message + "\"}\n",
                            status=httplib.BAD_REQUEST, content_type=content_type)

        if result == True:
            return Response(status=httplib.OK)
        else:
            return Response(response="{\"error\":\"Internal Server Error. "
                                     "Unable to contact with RabbitMQ process\"}\n",
                            status=httplib.INTERNAL_SERVER_ERROR,
                            content_type=content_type)

    # User submitted an unsupported Content-Type (only is valid application/json)
    else:
        return Response(response="{\"error\":\"Bad request. Content-type is not application/json\"}\n",
                        status=httplib.BAD_REQUEST,
                        content_type=content_type)


def process_request(request, tenantid, serverid):
    """Get the parsed contents of the form data

    :param string request:     The information of the received request
    :param string serverid:    the id of the monitored instance (server)

    :return: True
    """
    json = request.json
    if request.remote_addr:
        message = "[{}] received {} from ip {}:{}"\
            .format("-", json, request.environ[REMOTE_ADDR], request.environ[REMOTE_PORT])
    else:
        message = "[{}] received {} from test client"\
            .format("-", json)
    logging.info(message)

    key = ['contextResponses', 'contextElement', 'attributes']

    # Check that it contains the previous keys
    try:
        jsoncheck.checkit(json, key, 0)
    except NotFound as err:
        logging.error(err)
        raise err

    # Extract the list of attributes from the NGSI message
    attrlist = request.json[CONTEXT_RESPONSES][0][CONTEXT_ELEMENT][CONTEXT_ATTRIBUTES]

    data = list()

    for item in attrlist:
        name = item[CONTEXT_ATTRIBUTES_NAME]
        value = item[CONTEXT_ATTRIBUTES_VALUE]

        # Obtain the information of used memory and cpu
        if name == 'usedMemPct':
            verify_values(name, value)
            mem = float(value)
        elif name == 'cpuLoadPct':
            verify_values(name, value)
            cpu = float(value)
        elif name == 'netLoadPct':
            verify_values(name, value)
            net = float(value)
        elif name == 'freeSpacePct':
            verify_values(name, value)
            hdd = float(value)

    data.insert(len(data), cpu)
    data.insert(len(data), mem)
    data.insert(len(data), hdd)
    data.insert(len(data), net)

    # fix the first value of the list with the server identity
    data.insert(0, str(serverid))

    # fix the last value with the current date and time
    data.insert(len(fact_attributes) - 1, datetime.datetime.today().isoformat())

    # Check data coherency of time stamps
    # +1 is needed because the fact is not already added to the queue.
    # It checks that queue will have at least 2 facts.
    if len(mredis.range(tenantid, serverid)) + 1 >= 2:
        mredis.check_time_stamps(tenantid, serverid, mredis.range(tenantid, serverid), data)

    # Get the windowsize for the tenant from a redis queue
    windowsize = mredis.get_windowsize(tenantid)
    if windowsize == []:
        windowsize = myClotoDBClient.get_window_size(tenantid)
        mredis.insert_window_size(tenantid, windowsize)

     # Insert the result into the queue system
    mredis.insert(tenantid, serverid, data)
    logging.info(data)

    # If the queue has the number of facts defined by the windows size, it returns the
    # last window-size values (range) and calculates the media of them (in terms of memory and cpu)
    lo = mredis.media(mredis.range(tenantid, serverid), windowsize)

    # If the number of facts is lt window size, the previous operation returns a null lists
    if len(lo) != 0:
        try:
            rabbit = myqueue()
            if len(lo) == 1:
                lo.data = lo.data[0]
            message = "{\"serverId\": \"%s\", \"cpu\": %s, \"mem\": %s, \"hdd\": %s, \"net\": %s, \"time\": \"%s\"}" \
                      % (lo.data[0][1:-1], lo.data[1], lo.data[2], lo.data[3], lo.data[4], lo.data[5])

            logging_message = "[{}] sending message {}".format("-", message)

            logging.info(logging_message)

            # Send the message to the RabbitMQ components.
            result = rabbit.publish_message(tenantid, message)  # @UnusedVariable

        except Exception as ex:
            raise ex

    return True


def verify_values(name, value):
        """Checks if rule operands are expected strings and values are valid floats

        :param str name:        The name
        :param str value:       The value
        """

        myfloat = float(value)
        if myfloat < 0.0 or myfloat > 100.0:
            raise ValueError("Invalid value received for %s" % name)


def info(port):
    """Show some information about the execution of the process.
    """

    data = config.get('common', 'name')
    pid = os.getpid()

    logging.info("{} {}\n".format(data, __version__))
    logging.info("Running in stand alone mode")
    logging.info("Port: {}".format(port))
    logging.info("PID: {}\n".format(pid))
    logging.info("https://github.com/telefonicaid/fiware-facts\n\n\n")


def check_config_file():
    """Checks if configuration has mysql user with a user. If user parameter is empty, shows an error
    providing information about how to provide a valid settings file.
    The original Settings file could be found in facts_conf and it could be copied to the required folder.
    """
    if config.get('mysql', 'user') == '':
        logging.error("Cloto's Mysql data is empty. You should provide this information in the configuration file")
        logging.error("Please create a configuration file and add cloto MySql data to %s",
                      config.get('common', 'cfg_file_path'))
        logging.error("You can provide a Settings file in other locations using an environment variable called"
                      " FACTS_SETTINGS_FILE")

# process configuration file (if exists) and setup logging
if config.read(cfg_filename):
    logging.config.fileConfig(cfg_filename)
else:
    logging.basicConfig(stream=sys.stdout, level=cfg_defaults['logLevel'], format=cfg_defaults['logFormat'])

# Define the port of our server, by default 5000
port = config.getint('common', 'brokerPort')

# execute the flask server, WSGI server
http = WSGIServer(('', port), app)

# show general information about the execution of the process
info(port)
check_config_file()


def windowsize_updater():
    try:
        import pika
        connection = None
        connection = pika.BlockingConnection(pika.ConnectionParameters(
                host="localhost"))
        channel = connection.channel()

        channel.exchange_declare(exchange="windowsizes",
                                 exchange_type='direct')

        result = channel.queue_declare(exclusive=True)
        queue_name = result.method.queue

        channel.queue_bind(exchange="windowsizes",
                           queue=queue_name,
                           routing_key="windowsizes")

        logging.info('Waiting for windowsizes')

        def callback(ch, method, properties, body):
            try:
                logging.info("received windowsize: %s" % body)
                tenantid = body.split(" ")[0]
                windowsize = body.split(" ")[1]
                mredis.insert_window_size(tenantid, int(windowsize))

            except ValueError:
                logging.info("receiving an invalid body: " + body)

            except Exception as ex:
                logging.info("ERROR UPDATING WINDOWSIZE: " + ex.message)

        channel.basic_consume(callback,
                              queue=queue_name,
                              no_ack=True)

        channel.start_consuming()
    except Exception as ex:
        if ex.message:
            logging.error("Error %s:" % ex.message)
    finally:
        if connection == None:
            logging.error("There is no connection with RabbitMQ. Please, check if it is alive")
        else:
            connection.close()

gevent.spawn(windowsize_updater)


def start_server():
    http.serve_forever()

if __name__ == '__main__':
    start_server()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2014 Telefónica Investigación y Desarrollo, S.A.U
#
# This file is part of FI-WARE project.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#
# For those usages not covered by the Apache version 2.0 License please
# contact with opensource@tid.es
#
__author__ = 'gjp'

"""File with Constants used in fiware-facts project.
"""

# HTTP CONSTANTS
CONTENT_HEADER = u'content-type'
JSON_TYPE = u'application/json'
REMOTE_ADDR = u'REMOTE_ADDR'
REMOTE_PORT = u'REMOTE_PORT'

# ORION CONSTANTS
CONTEXT_RESPONSES = u'contextResponses'
CONTEXT_ELEMENT = u'contextElement'
CONTEXT_ATTRIBUTES = u'attributes'
CONTEXT_ATTRIBUTES_NAME = u'name'
CONTEXT_ATTRIBUTES_VALUE = u'value'



# -*- encoding: utf-8 -*-
#
# Copyright 2014 Telefonica Investigación y Desarrollo, S.A.U
#
# This file is part of FI-WARE project.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#
# For those usages not covered by the Apache version 2.0 License please
# contact with opensource@tid.es
#
__author__ = 'fla'

from config import config
from pika import exceptions
import pika
import logging
import time

"""Class to notify the facts to the RabbitMQ queue to be taken afterwards
by the cloto component.
"""


class myqueue(object):

    def __init__(self):
        """ Initialize the class and create a connection with a RabbitMQ server instance.
        """
        # Get the default IP address of the RabbitMQ server.
        rabbitmq = config.get('common', 'rabbitMQ')
        self.connection = None
        self.channel = None

        # Open a remote connection to RabbitMQ on specified IP address
        try:
            self.connection = pika.BlockingConnection(pika.ConnectionParameters(
                host=rabbitmq))

            # Open the channel
            self.channel = self.connection.channel()
        except (pika.exceptions.AMQPConnectionError, pika.exceptions.AMQPChannelError), error:
            logging.error('Exception while connecting to Rabbit %s' % error)
            raise Exception("Error with Rabbit connection")

    def publish_message(self, tenantid, message):
        """Publish a message related to a tenantid in the rabbitmq and
        close the connection

        :param str tenantid:      The id of the tenant
        :param str message:       The well-formatted message to send
        """
        if self.channel:
            self.channel.exchange_declare(exchange="facts",
                                 exchange_type='direct')

            try:
                # Send a message
                self.channel.basic_publish(exchange="facts",
                                           routing_key=tenantid,
                                           body=message)

                logging_message = "[{}] Sent message to RabbitMQ".format("-")

                logging.info(logging_message)
            except (pika.exceptions.AMQPConnectionError, pika.exceptions.AMQPChannelError), error:
                logging.error('AMQP Connection failed. Trying again... %s' % error)
                raise Exception("AMQP Connection failed.")
        else:
            logging.error('AMQP channel not properly created...')
            raise Exception("AMQP channel not properly created...")

        if self.connection and self.connection.is_open:
            # Close the connection
            self.connection.close()
        else:
            logging.error('AMQP connection not properly created...')

            raise Exception("AMQP connection not properly created...")



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2014 Telefónica Investigación y Desarrollo, S.A.U
#
# This file is part of FI-WARE project.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#
# For those usages not covered by the Apache version 2.0 License please
# contact with opensource@tid.es
#
__author__ = 'gjp'
import logging.config
from config import config
import MySQLdb as mysql
from keystoneclient.exceptions import NotFound


class cloto_db_client():
    """This class provides methods to provide connection with Cloto database.
    """
    conn = None

    def get_window_size(self, tenantId):
        """
        This method is in charge of retrieve the window size of a tenantId from cloto database.

        :param tenantId: the id of the tenant to request the windowsize
        :return: the window size
        """
        try:
            if self.conn == None:
                self.conn = mysql.connect(charset=config.get('mysql', 'charset'), use_unicode=True,
                                     host=config.get('mysql', 'host'),
                                     user=config.get('mysql', 'user'), passwd=config.get('mysql', 'password'),
                                     db=config.get('mysql', 'db'))
            cursor = self.conn.cursor()
            cursor.execute('SELECT * FROM cloto.cloto_tenantinfo WHERE tenantId="%s"' % tenantId)
            data = cursor.fetchall()
            if len(data) == 0:
                raise NotFound('{"error": "TenantID %s not found in cloto database"}' % tenantId)
            else:
                tenant_information = data[0]
                window_size = tenant_information[1]

        except Exception, e:
                logging.error("Error %s" % e.message)
                raise e
        finally:
            if self.conn:
                self.conn.close()
                self.conn = None
        return window_size



# -*- encoding: utf-8 -*-
#
# Copyright 2014 Telefónica Investigación y Desarrollo, S.A.U
#
# This file is part of FI-WARE project.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#
# For those usages not covered by the Apache version 2.0 License please
# contact with opensource@tid.es
#

__author__ = 'fla'
__version__ = '2.5.0'

from ConfigParser import SafeConfigParser
from sys import platform as _platform
import platform
import os.path
import datetime


"""
Default configuration.

The configuration `cfg_defaults` are loaded from `cfg_filename`, if file exists in
/etc/fiware.d/fiware-facts.cfg

Optionally, user can specify the file location manually using an Environment variable called FACTS_SETTINGS_FILE.
"""

name = 'fiware-facts'

cfg_dir = "/etc/fiware.d"

if os.environ.get("FACTS_SETTINGS_FILE"):
    cfg_filename = os.environ.get("FACTS_SETTINGS_FILE")

else:
    cfg_filename = os.path.join(cfg_dir, '%s.cfg' % name)

cfg_defaults = {
    'brokerPort':   5000,                   # port of our facts broker
    'redisPort':    6379,                   # port of Redis
    'clotoPort':    8000,                   # port of fiware-cloto
    'clotoVersion': 'v1.0',                 # Cloto API version
    'redisHost':    'localhost',            # host of Redis
    'redisQueue':   'policymanager',        # name of the queue in redis
    'rabbitMQ':     'localhost',            # IP of the RabbitMQ server
    'cloto':        '127.0.0.1',            # IP of fiware-cloto component
    'clotoVersion': 'v1.0',                 # Cloto API version
    'name':         'policymanager.facts',  # name of the server
    'logLevel':     'INFO',
    'logFormat':    '%(asctime)s %(levelname)s policymanager.facts %(message)s',
    'maxTimeWindowsize':   10               # The maximum time between facts to be considered a valid Fact (seconds).
}

cfg_mysql_defaults = {
    'host':   'localhost',                   # host of Cloto Mysql database
    'charset':    'utf8',                   # charset of the data base
    'user':    '',                   # user of cloto database
    'password': '',                 # password for the user
    'db':    'cloto',            # name of the cloto dataBase, default: cloto
}

cfg_loggers_defaults = {
    'keys':   'root'
}

cfg_handlers_defaults = {
    'keys':   'console, file'
}

cfg_formatters_defaults = {
    'keys':   'standard',
}

cfg_formatter_standard_defaults = {
    'class':   'logging.Formatter',
    'format':    '%(asctime)s %(levelname)s policymanager.facts %(message)s',
}

cfg_logger_root_defaults = {
    'level':   'INFO',
    'handlers':    'console, file'
}

cfg_handler_console_defaults = {
    'level':   'DEBUG',
    'class':    'StreamHandler',
    'formatter':    'standard',
    'args': '(sys.stdout,)',
}

cfg_handler_file_defaults = {
    'level':   'DEBUG',
    'class':    'handlers.RotatingFileHandler',
    'formatter':    'standard',
    'logFilePath':    '/var/log/fiware-facts',
    'logFileName':    'fiware-facts.log',
    'logMaxFiles':    3,
    'logMaxSize':    '5*1024*1024  ; 5 MB',
    'args': "('%(logFilePath)s/%(logFileName)s', 'a', %(logMaxSize)s, %(logMaxFiles)s)",
}

config = SafeConfigParser(cfg_defaults)

# Create the common section in the same way that we have in the configuration file: fiware-facts.cfg
config.add_section('common')
config.add_section('mysql')
config.add_section('loggers')
config.add_section('handlers')
config.add_section('formatters')
config.add_section('formatter_standard')
config.add_section('logger_root')
config.add_section('handler_console')
config.add_section('handler_file')


# Initialize the content of the config parameters
for key, value in cfg_defaults.items():
    config.set('common', key, str(value))

for key, value in cfg_mysql_defaults.items():
    config.set('mysql', key, str(value))

for key, value in cfg_loggers_defaults.items():
    config.set('loggers', key, str(value))

for key, value in cfg_handlers_defaults.items():
    config.set('handlers', key, str(value))

for key, value in cfg_formatters_defaults.items():
    config.set('formatters', key, str(value))

for key, value in cfg_formatter_standard_defaults.items():
    config.set('formatter_standard', key, str(value))

for key, value in cfg_logger_root_defaults.items():
    config.set('logger_root', key, str(value))

for key, value in cfg_handler_console_defaults.items():
    config.set('handler_console', key, str(value))

for key, value in cfg_handler_file_defaults.items():
    config.set('handler_file', key, str(value))

config.set('common', 'cfg_file_path', str(cfg_filename))

windowsize_facts = datetime.timedelta(seconds=int(config.get('common', 'maxTimeWindowsize')))

fact_attributes = ['serverId', 'CpuValue', 'MemValue', 'HddValue', 'NetValue', 'DataTime']

windowsize_attributes = ['tenantId', 'value']



# -*- encoding: utf-8 -*-
#
# Copyright 2014 Telefonica Investigación y Desarrollo, S.A.U
#
# This file is part of FI-WARE project.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#
# For those usages not covered by the Apache version 2.0 License please
# contact with opensource@tid.es
#
__author__ = 'fla'

""" Class to verify the content of the json message received in the message
"""


class jsoncheck(object):
    @classmethod
    def checkit(self, dictionary, keylist, cycle):
        """ Check recursively the json dictionary in order to check that it
        contains the keys included in the keylist.

        :param dict dictionary   The dictionary of the json message
        :param list keylist      The list of keys to be checked
        :param int cycle         The step in the recurrent algorith

        :return                  By default this method does not return
                                 anything, except when some key is not found
                                 in the dictionary, then return exception.
        """
        if len(keylist) != 0:
            # Get the first key of the list, in a list format
            key = keylist[:1]

            # Get the list with the rest of keys
            restkey = keylist[1:]

            # Check the dictionary in order to detect if the key is there,
            # in that case it returns the value of that key.
            # Otherwise, it returns a null list '[]'
            result = reduce(lambda x, y:
                            dictionary.get(y) and x.append(dictionary[y]) or x, key, [])

            # If it cannot find the key in the dictionary
            if result == []:
                # Error, there is no key in the dictionary
                raise Exception("Invalid json message. We cannot obtain the key: {}".format(key[0]))

            # The first time, I need to extract the next dictionary in a different way
            if cycle == 0:
                if isinstance(result[0], list):
                    result = result[0][0]
                else:
                    # Error, there is no key in the dictionary
                    raise Exception("Invalid json message. We expected "
                                    "'['contextResponses', 'context', 'attributes']' keys but obtain"
                                    " only '{}' key".format(key[0]))
            else:
                result = result[0]

            self.checkit(result, restkey, cycle + 1)



# -*- encoding: utf-8 -*-
#
# Copyright 2014 Telefonica Investigación y Desarrollo, S.A.U
#
# This file is part of FI-WARE project.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#
# For those usages not covered by the Apache version 2.0 License please
# contact with opensource@tid.es
#
__author__ = 'fla'

from config import config, fact_attributes, windowsize_attributes
from mylist import mylist
from redis.exceptions import ConnectionError
import logging
import redis

nqueue = config.get('common', 'redisQueue')

# TODO: change the default queue in redis to incorporate the server ID.
# TODO: Check the temporal mark when we want to calculate the media of data.
# TODO: Review Kalman filter in order to change the estimation of the data in the windows size.


class myredis(object):
    """
    Class to create lists on redis distribution. it is assumed
    that a redis-server is running at host:port.
    """
    def __init__(self):
        """ Initialize the class and create a connection with a redis server instance.
        The initialization includes the deletion of the previous queue
        """
        host = config.get('common', 'redisHost')
        port = config.getint('common', 'redisPort')
        self.r = redis.StrictRedis(host=host, port=port, db=0)

        try:
            # TODO: We need to delete all the queues.
            self.r.flushall()
            #self.r.delete(nqueue)

        except ConnectionError:
            message = "[{}] Cannot delete the list. Possibly redis is down".format("-")
            logging.error(message)

    def insert(self, tenantid, serverid, data):
        """ Insert data into the redis queue.

        :param list data:     The list of data to be stored in the queue
        :return               This operation does not return anything except when the data
                              is no list or the number of element is not equal to 4.
        """
        windowsize = self.get_windowsize(tenantid)
        if isinstance(windowsize, list) and len(windowsize) == 1:
            windowsize = int(windowsize[0])
        if isinstance(data, list) and len(data) == len(fact_attributes):
            self.r.rpush(tenantid + "." + serverid, data)
            self.r.ltrim(tenantid + "." + serverid, -int(windowsize), -1)
        else:
            return "error"

    def range(self, tenantid, serverid):
        """ Return the list of element stored the que queue.
         :return a list of lists
        """
        return self.r.lrange(tenantid + "." + serverid, -100, 100)

    def media(self, lista, windowsize):
        """ Calculate the media of a list of data

         :param mylist lista     The mylist instance with the data to be added.
         :return mylist          The media of the data
        """
        if isinstance(windowsize, list) and len(windowsize) == 1:
            windowsize = int(windowsize[0])
        if len(lista) >= windowsize:
            return self.sum(lista) / len(lista)
        else:
            result = mylist()
            result.data = []
            return result

    def sum(self, lista):
        """ Calculate the sum of a list of data

        :param mylist lista:     The lists of lists which will be added.
        return mylist            The new mylist instance with the result of the operation.
        """
        if len(lista) > 1:
            return mylist.sum(lista)
        elif len(lista) == 1:
            r1 = mylist()
            r1.insert(lista)
            return r1
        else:
            return '[]'

    def delete(self):
        """ Delete a specific queue from the redis system.
        """
        self.r.delete(nqueue)

    def insert_window_size(self, tenantid, data):
        """ Insert data into the redis queue.

        :param list data:     The list of data to be stored in the queue
        :return               This operation does not return anything except when the data
                              is no list or the number of element is not equal to 4.
        """
        if isinstance(data, int) or isinstance(data, long):
            self.r.rpush("windowsize" + "." + tenantid, data)
            self.r.ltrim("windowsize" + "." + tenantid, -1, -1)

    def get_windowsize(self, tenantid):
        """ Return the list of element stored the que queue for the tenant.
         :return a list of lists
        """
        return self.r.lrange("windowsize" + "." + tenantid, -100, 100)

    def check_time_stamps(self, tenantid, serverid, lista, data):
        """
        Check if the list is valid checking last item time-stamp with the new item time-stamp
        :return:
        """
        from dateutil import parser
        textmin = lista[-1].split("'")
        datemin = parser.parse(textmin[-2], fuzzy=True)
        datemax = parser.parse(data[-1], fuzzy=True)
        from config import windowsize_facts

        timediff = datemax - datemin
        if  timediff > windowsize_facts:
            self.r.delete(tenantid + "." + serverid)
            return False
        else:
            return True



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2015 Telefónica Investigación y Desarrollo, S.A.U
#
# This file is part of FI-WARE project.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#
# For those usages not covered by the Apache version 2.0 License please
# contact with opensource@tid.es
#
from setuptools import setup, find_packages
from pip.req import parse_requirements
from facts.config import __version__

# parse_requirements() returns generator of pip.req.InstallRequirement objects
install_reqs = parse_requirements("requirements.txt", session=False)
# > requirements_list is a list of requirement; e.g. ['requests==2.6.0', 'Fabric==1.8.3']
requirements_list = [str(ir.req) for ir in install_reqs]

setup(
  name='fiware-facts',
  packages=find_packages(exclude=['tests*']),
  install_requires=requirements_list,
  package_data={
    'facts_conf': ['*.cfg']
  },
  version=__version__,
  description='Server to process the incoming facts from the Orion Context Broker',
  author='Fernando Lopez Aguilar, Guillermo Jimenez Prieto',
  author_email='fernando.lopezaguilar@telefonica.com, e.fiware.tid@telefonica.com',
  license='Apache 2.0',
  url='https://github.com/telefonicaid/fiware-facts',
  download_url='https://github.com/telefonicaid/fiware-facts/tarball/v%s' % __version__,
  keywords=['fiware', 'policy', 'manager', 'cloud'],
  classifiers=[
        "License :: OSI Approved :: Apache Software License", ],
)



# -*- encoding: utf-8 -*-
#
# Copyright 2014 Telefonica Investigación y Desarrollo, S.A.U
#
# This file is part of FI-WARE project.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#
# For those usages not covered by the Apache version 2.0 License please
# contact with opensource@tid.es
#
__author__ = 'fla'

""" Specific list class in order to work with the appropriate representation
of message data.
"""


class mylist(object):
    def __init__(self, data=None):
        """Initialize the list of data

        :param list   data:     The initial value of the list to be processed

        :return:                The mylist object created
        """
        self.insert(data)

    @staticmethod
    def parselist(data):
        """ Static method used to obtain a list from a string representation
        of the data

        :param string data:   The string representation of data

        :return:              The list representation of the string
        """
        # Delete from the string the '[', ']'  and white spaces and divide the
        # content by ',' in a list
        p3 = data.lstrip().replace('[', '').replace(']', '').split(',')

        aux = len(p3) - 1
        # Convert the %cpu and %memory into a float value
        for i in range(1, aux):
            p3[i] = float(p3[i])

        # delete white spaces and ''' in the last value
        p3[aux] = str(p3[aux].lstrip().replace("'", ""))

        return p3

    def insert(self, data):
        """ Insert a list into the class

        :param list data:   The list to be inserted into the class

        :return:            Nothing
        """
        # If the parameter data is not a list, assign a null list
        if isinstance(data, list) == False:
            self.data = []
        else:
            # If the 1st element is a list too e.g.:[ [1, 2, 4], [1, 2, 4] ...]
            # OR the 1st element in a integer number e.g. [1, 2 ,4]
            # OR (the 1st is a string AND the 1st character is a '[' e.g. ['[1, 2, 4]', '[1, 2, 4]', ...]
            # then data is a valid list to be processed
            if isinstance(data[0], list) or isinstance(data[0], int) or isinstance(data[0], float) or \
                    (isinstance(data[0], str) and data[0][0] != '['):
                self.data = data
            else:
                # if the 1st element is a string BUT the 1st character is != to a '['
                # e.g. ['['1', 2.3, 'xxxx']', '['1', 2.3, 'xxxx']', '['1', 2.3, 'xxxx']']
                if isinstance(data[0], str):
                    p4 = []

                    # Extract the sublist in order to process them
                    for i in range(0, len(data)):
                        p3 = mylist.parselist(data[i])

                        p4.append(p3)

                    self.data = p4

    def delete(self):
        """ Delete the content of the list
        """
        self.data = []

    @staticmethod
    def sum(data):
        """ Calculate the addition of all the sublists received in the data list
         :param list data:      The list of data to add with the format [ [1, 2, 'xxx'], [4, 5, 'yyy], ... ]

         :return:               A list with the sum all data in 2nd and 3rd position of each list
        """
        aux = mylist(data).get()
        if len(aux) > 1:
            return mylist(aux[0]) + mylist.sum(data[1:])
        elif len(aux) == 1:
            return mylist(aux[0])
        else:
            return float(0)

    def __len__(self):
        """ Calculate the len of teh data list
         :return:    The len of the data list contained in my class
        """
        return len(self.data)

    def __add__(self, other):
        """ Operator overloading: add the values of the list except the first one
        :param mylist other:      An instance of the mylist class whose data is someth. similar to [1, 2, 'xxxx']
        :return mylist:           A new mylist instance with the sum of all components except the first and the last
        """
        if isinstance(other, mylist):
            result = mylist()
            result.data = self.data

            limit = len(self.data) - 1

            for i in range(1, limit):
                result.data[i] = self.data[i] + other.data[i]

            result.data[limit] = other.data[limit]

            return result

    def __div__(self, other):
        """ Operator overloading: division of the list by a integer number
        :param int other:       The divisor of the list (except the last value). It is not necessary to check if it's 0
                                due to it will be manage by the caller of the function.
        :return mylist:         A new mylist instance with the result of the division.
        """
        if isinstance(other, int):
            result = mylist(self.data)
            for i in range(1, len(self.data) - 1):
                result.data[i] = self.data[i] / float(other)

            return result

    def get(self):
        """ Return the list
        """
        return self.data

    @classmethod
    def isdata(self, info):
        """ Check that info is a valid mylist and the content of data.data is a list
        :param <?> data:       The information to validate
        :return boolean:       True if it is a valid information
                                    info is a instance of mylist
                                    info.get() return a list AND
                                    len(info) is 3
                               False otherwise
        """
        if isinstance(info, mylist) and isinstance(info.get(), list):
            if len(info) == 3:
                return True
            else:
                return False
        else:
            return False
